﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using ComPDFKit.NativeMethod;
using ComPDFKit_Conversion.Converter;
using PDF_Office.CustomControl;
using PDF_Office.CustomControl.SystemControl;
using PDF_Office.ViewModels;
using PDF_Office.Views;
using PDF_Office.Views.BOTA;
using PDF_Office.Views.Dialog;
using PDF_Office.Views.HomePanel;
using Prism.DryIoc;
using Prism.Ioc;
using Prism.Regions;
using PDF_Office.Model;
using PDF_Office.Views.PageEdit;
using PDF_Office.Properties;
using PDFSettings.Settings;
using PDF_Office.Views.Tools;
using PDF_Office.Views.Dialog.ToolsDialogs.SaftyDialogs;
using PDF_Office.Views.Dialog.ToolsDialogs.CompressDialogs;
using PDF_Office.Views.Dialog.PageEditDialogs;
using PDF_Office.Helper;
using PDF_Office.Views.Dialog.HomePageToolsDialogs;
using PDF_Office.Views.Dialog.HomePageToolsDialogs.HomePagePrinter;
using PDF_Office.Views.PropertyPanel;
using PDF_Office.Views.Dialog.BOTA;
using PDF_Office.Views.PropertyPanel.AnnotPanel;
using PDF_Office.Views.FillAndSign;
using PDF_Office.Views.FillAndSign.PropertyPanel;

namespace PDF_Office
{
    /// <summary>
    /// App.xaml 的交互逻辑
    /// </summary>
    public partial class App : PrismApplication
    {
        public static string CurrentPath = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), "PDF Office");

        public static MainWindowViewModel mainWindowViewModel;

        public static List<string> OpenedFileList = new List<string>();

        public static bool IsFirstOpen = true;

        public App()
        {
#if !DEBUG
            //接入Appcenter后台数据检测
            string appkey = "b099dccc-1ca5-4ea1-a2fa-afd4a6e1f8bb"
            AppCenter.Start(appkey, typeof(Analytics), typeof(Crashes));
            var countryCode = RegionInfo.CurrentRegion.TwoLetterISORegionName;//上传国家信息
            AppCenter.SetCountryCode(countryCode);
#endif
            LicenseVerify();

            InitSettings();

            try
            {
                DirectoryInfo info = new DirectoryInfo(CurrentPath);
                if (File.Exists(CurrentPath))
                    File.Delete(CurrentPath);
                if (!info.Exists)
                {
                    info.Create();
                }
            }
            catch (Exception ex)
            {
            }
        }

        private void InitSettings()
        {
            if (Settings.Default.RecentOpenFiles == null)
                Settings.Default.RecentOpenFiles = new RecentOpenFiles();

            if (Settings.Default.AppProperties == null)
                Settings.Default.AppProperties = new PDFSettings.APPSettingProperties();
        }

        /// <summary>
        /// 创建启动窗口
        /// </summary>
        /// <returns></returns>
        protected override Window CreateShell()
        {
            return Container.Resolve<MainWindow>();
        }

        protected override void RegisterTypes(IContainerRegistry containerRegistry)
        {
            #region 注册内容组件
            containerRegistry.RegisterForNavigation<HomeContent>();
            containerRegistry.RegisterForNavigation<ViewContent>();
            containerRegistry.RegisterForNavigation<MainContent>();
            containerRegistry.RegisterForNavigation<HomeCloudContent>("Cloud");
            containerRegistry.RegisterForNavigation<HomeToolsContent>("Tools");
            containerRegistry.RegisterForNavigation<HomeGuidContent>("Guid");
            containerRegistry.RegisterForNavigation<BOTAContent>();
            containerRegistry.RegisterForNavigation<PropertyPanelContent>();
            containerRegistry.RegisterForNavigation<PageEditContent>();
            containerRegistry.RegisterForNavigation<BottomToolContent>();
            containerRegistry.RegisterForNavigation<ToolsBarContent>();
            containerRegistry.RegisterForNavigation<AnnotToolContent>();
            containerRegistry.RegisterForNavigation<BookmarkContent>();
            containerRegistry.RegisterForNavigation<HomePagePrinterModSizeContent>();
            containerRegistry.RegisterForNavigation<HomePagePrinterModPosterContent>();
            containerRegistry.RegisterForNavigation<HomePagePrinterModMutipleContent>();
            containerRegistry.RegisterForNavigation<FreehandAnnotProperty>(); 
            containerRegistry.RegisterForNavigation<FreetextAnnotProperty>();
            containerRegistry.RegisterForNavigation<ImageAnnotProperty>();
            containerRegistry.RegisterForNavigation<LinkAnnotProperty>(); 
            containerRegistry.RegisterForNavigation<SharpsAnnotProperty>();
            containerRegistry.RegisterForNavigation<StampAnnotProperty>();
            containerRegistry.RegisterForNavigation<TextAnnotProperty>();
            containerRegistry.RegisterForNavigation<TextEditProperty>();
            //填写与签名
            containerRegistry.RegisterForNavigation<FillAndSignContent>();
            containerRegistry.RegisterForNavigation<DateFillProperty>();
            containerRegistry.RegisterForNavigation<ShapFillProperty>();
            containerRegistry.RegisterForNavigation<TextFillProperty>();
            #endregion

            #region 注册弹窗
            containerRegistry.RegisterDialog<VerifyPassWordDialog>(DialogNames.VerifyPassWordDialog);
            containerRegistry.RegisterDialog<FullScreenWindow>(DialogNames.FullScreenDialog);
            containerRegistry.RegisterDialog<CompressDialog>(DialogNames.CompressDialog);
            containerRegistry.RegisterDialog<SetPasswordDialog>(DialogNames.SetPasswordDialog);
            containerRegistry.RegisterDialog<CheckPasswordDialog>(DialogNames.CheckPasswordDialog);
            containerRegistry.RegisterDialog<DeleteSafetySettingsDialog>(DialogNames.DeleteSafetySettingsDialog);
            containerRegistry.RegisterDialog<CompressProgressBarDialog, PDF_Office.ViewModels.Dialog.ToolsDialogs.CompressDialogs.CompresProgressBarDialogViewModel>(DialogNames.CompressProgressBarDialog);
            containerRegistry.RegisterDialog<InsertDialog>(DialogNames.InsertDialog);
            containerRegistry.RegisterDialog<SplitDialog>(DialogNames.SplitDialog);
            containerRegistry.RegisterDialog<ExtractDialog>(DialogNames.ExtractDialog);
            containerRegistry.RegisterDialog<HomePageExtractDialog>(DialogNames.HomePageExtractDialog);
            containerRegistry.RegisterDialog<HomePageInsertDialog>(DialogNames.HomePageInsertDialog);
            containerRegistry.RegisterDialog<HomePageSplitDialog>(DialogNames.HomePageSplitDialog);
            containerRegistry.RegisterDialog<HomePagePrinterDialog>(DialogNames.HomePagePrinterDialog);
            containerRegistry.RegisterDialog<HomePagePrinterPageSetDialog>(DialogNames.HomePagePrinterPageSetDialog);
            containerRegistry.RegisterDialog<HomePagePictureToPDFDialog>(DialogNames.HomePagePictureToPDFDialog);
            containerRegistry.RegisterDialog<AddBookmarkDialog>(DialogNames.AddBookmarkDialog);
            #endregion
        }

        protected override void ConfigureRegionAdapterMappings(RegionAdapterMappings regionAdapterMappings)
        {
            var region = new SingleActiveRegion() { Name = RegionNames.MainRegion };

            region.Behaviors.Add(DragablzWindowBehavior.BehaviorKey, new DragablzWindowBehavior());

            Container.Resolve<IRegionManager>().Regions.Add(region);

            base.ConfigureRegionAdapterMappings(regionAdapterMappings);
        }

        /// <summary>
        /// 验证SDK和转档库密钥
        /// </summary>
        /// <returns></returns>
        private static bool LicenseVerify()
        {
            bool result = false;
            try
            {
                result = CPDFSDKVerifier.LoadNativeLibrary();
                if (!result)
                    return false;
            }
            catch { }
            SDKLisenceHelper sdkhelper = new SDKLisenceHelper();
            string devKey = "";
            string devSecret = "";
#if DEBUG
            devKey = sdkhelper.DevKey;
            devSecret = sdkhelper.DevSecret;
#endif
            string userKey = sdkhelper.UserKey;
            string userSecret = sdkhelper.UserSecret;

            try
            {
                CPDFSDKVerifier.LicenseErrorCode verifyResult = CPDFSDKVerifier.LicenseVerify(devKey, devSecret);
                if (verifyResult != CPDFSDKVerifier.LicenseErrorCode.LICENSE_ERR_SUCCESS)
                    return false;
            }
            catch { }

            string ConverterDevKey = sdkhelper.ConverterDevKey;
            string ConverterDevSecret = sdkhelper.ConverterDevSecret;
            string ConverterUserkey = sdkhelper.ConverterUserkey;
            string ConverterUserSecret = sdkhelper.ConverterUserSecret;

            try
            {
                /*LicenseError licenseerror = CPDFConverter.SDKLicenseVerify(ConverterDevKey, ConverterDevSecret, ConverterUserkey, ConverterUserSecret);
            string resPath = Path.GetDirectoryName(typeof(MainWindow).Assembly.Location) + "\\";
            if (licenseerror != LicenseError.ERR_SUCCESS)
            {
                return false;
            }
                CPDFConverter.Init(resPath);*/
                string resPath = Path.GetDirectoryName(typeof(MainWindow).Assembly.Location) + "\\";
                LicenseError licenseerror = CPDFConverter.LicenseVerify(ConverterDevKey, ConverterDevSecret, ConverterUserkey, ConverterUserSecret);
                if (licenseerror != LicenseError.ERR_SUCCESS)
                {
                    //MessageBox.Show("ComPDFKit Conversion SDK Load Failed!");
                    return false;
                }
                CPDFConverter.Init(resPath);
            }
            catch { }
            return result;
        }

        protected override void OnExit(ExitEventArgs e)
        {
            Environment.Exit(1);
            base.OnExit(e);
        }
    }
}