﻿using ComPDFKitViewer.AnnotEvent;
using PDF_Office.Model;
using PDF_Office.ViewModels.Tools;
using PDFSettings;
using Prism.Commands;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;

namespace PDF_Office.ViewModels.PropertyPanel.AnnotPanel
{
    
    public class TextAnnotPropertyViewModel:BindableBase, INavigationAware
    {
     

        #region 属性

        private AnnotArgsType annotType;
        public AnnotArgsType AnnotType
        {
            get { return annotType; }
            set
            {
                SetProperty(ref annotType, value);
                SetAnnotType();
            }
        }


        private string annotTypeTitle;
        public string AnnotTypeTitle
        {
            get { return annotTypeTitle; }
            set
            {
                SetProperty(ref annotTypeTitle, value);
            }
        }


        private double annotOpacity;
        public double AnnotOpacity
        {
            get {return annotOpacity;}
            set 
            { 
                SetProperty(ref annotOpacity, value);
            }
        }


        /// <summary>
        /// 示例背景
        /// </summary>
        private Brush sampleTextBg = new SolidColorBrush(Colors.Transparent);
        public Brush SampleTextBg
        {
            get { return sampleTextBg; }
            set { SetProperty(ref sampleTextBg, value); SetAnnotProperty(); }
        }


        private Brush selectColor = new SolidColorBrush(Colors.Transparent);
        public Brush SelectColor
        {
            get { return selectColor; }
            set { SetProperty(ref selectColor, value); SetAnnotProperty(); }
        }

        #endregion

        private AnnotHandlerEventArgs Annot;
        private AnnotPropertyPanel PropertyPanel;
        public DelegateCommand<object> SelectedColorChangedCommand { get; set; }
        public DelegateCommand<object> OpacityItemCommand { get; set; }
        public DelegateCommand<object> OpacityValueChangedCommand { get; set; }

        public TextAnnotPropertyViewModel()
        {
            SelectedColorChangedCommand = new DelegateCommand<object>(SelectedColorChanged_Click);
            OpacityItemCommand = new DelegateCommand<object>(OpacityItemCommand_Click);
            OpacityValueChangedCommand = new DelegateCommand<object>(OpacityValueChanged_Command);
        }

        private void OpacityItemCommand_Click(object obj)
        {
            if(obj!= null)
            {
                int opacity = -1;
                if(int.TryParse((string)obj, out opacity))
                {
                    AnnotOpacity = opacity/100D;
                    Dictionary<AnnotArgsType, object> changeData = new Dictionary<AnnotArgsType, object>();
                    changeData[AnnotType] = AnnotOpacity;
                    PropertyPanel.DataChangedInvoke(this, changeData);
                }
            }
        }

        private void OpacityValueChanged_Command(object obj)
        {
            if (obj != null)
            {
                if (AnnotType == AnnotArgsType.AnnotHighlight)
                    SampleTextBg.Opacity = annotOpacity;
                else
                    SelectColor.Opacity = annotOpacity;

                SetAnnotProperty();
                Dictionary<AnnotArgsType, object> changeData = new Dictionary<AnnotArgsType, object>();
                changeData[AnnotType] = annotOpacity;
                PropertyPanel.DataChangedInvoke(this, changeData);
            }
        }

        public void SelectedColorChanged_Click(object color)
        {
            if (color != null)
            {
                var colorValue = (Color)color;
                if (colorValue != null)
                {
                   
                    switch (AnnotType)
                    {
                        case AnnotArgsType.AnnotHighlight:
                            SampleTextBg = new SolidColorBrush(colorValue);
                            SampleTextBg.Opacity = AnnotOpacity;
                            break;
                        case AnnotArgsType.AnnotUnderline:
                        case AnnotArgsType.AnnotStrikeout:
                        case AnnotArgsType.AnnotSquiggly:
                        case AnnotArgsType.AnnotSticky:
                            SelectColor = new SolidColorBrush(colorValue);
                            SelectColor.Opacity = AnnotOpacity;
                            break;
                    }

                    Dictionary<AnnotArgsType, object> changeData = new Dictionary<AnnotArgsType, object>();
                    changeData[AnnotType] = color;
                    PropertyPanel.DataChangedInvoke(this, changeData);
                }
            }
        }

        private void SetAnnotType()
        {
            switch (AnnotType)
            {
                case AnnotArgsType.AnnotHighlight:
                    AnnotTypeTitle = "高亮";
                    break;
                case AnnotArgsType.AnnotUnderline:
                    AnnotTypeTitle = "下划线";
                    break;
                case AnnotArgsType.AnnotStrikeout:
                    AnnotTypeTitle = "删除线";
                    break;
                case AnnotArgsType.AnnotSquiggly:
                    AnnotTypeTitle = "波浪线";
                    break;
                case AnnotArgsType.AnnotSticky:
                    AnnotTypeTitle = "？？";
                    break;
            }
        }

        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }


       
        public void OnNavigatedTo(NavigationContext navigationContext)
        {

            navigationContext.Parameters.TryGetValue<AnnotPropertyPanel>(ParameterNames.PropertyPanelContentViewModel, out PropertyPanel);
          //  navigationContext.Parameters.TryGetValue<ComPDFKitViewer.PdfViewer.CPDFViewer>(ParameterNames.PDFViewer, out PDFViewer);
            if(PropertyPanel != null)
            {
                Annot = PropertyPanel.annot;
                AnnotType = Annot.EventType;
                GetAnnotProperty();
            }

        }


        private void GetAnnotProperty()
        {
            if (Annot != null)
            {
                switch (Annot.EventType)
                {
                    case AnnotArgsType.AnnotHighlight:
                      if (Annot is TextHighlightAnnotArgs)
                        {
                            var Hightlight = Annot as TextHighlightAnnotArgs;
                            AnnotOpacity = Hightlight.Transparency;
                            SampleTextBg = new SolidColorBrush(Hightlight.Color);

                        }
                        break;

                    case AnnotArgsType.AnnotUnderline:
                        {
                            var Underline = Annot as TextUnderlineAnnotArgs;
                            AnnotOpacity = Underline.Transparency;
                             SampleTextBg = new SolidColorBrush(Colors.Transparent);
                            SelectColor = new SolidColorBrush(Underline.Color);
                        }
                        break;

                    case AnnotArgsType.AnnotSquiggly:
                        {
                            var Squiggly = Annot as TextSquigglyAnnotArgs;
                            AnnotOpacity = Squiggly.Transparency;
                            SampleTextBg = new SolidColorBrush(Colors.Transparent);
                            SelectColor = new SolidColorBrush(Squiggly.Color);
                        }
                        break;

                }
            }
        }


        private void SetAnnotProperty()
        {
            if (Annot != null)
            {
                switch (Annot.EventType)
                {
                    case AnnotArgsType.AnnotHighlight:
                        if (Annot is TextHighlightAnnotArgs)
                        {
                            var Hightlight = Annot as TextHighlightAnnotArgs;

                            if (Hightlight.Transparency != AnnotOpacity)
                                Hightlight.Transparency = AnnotOpacity;

                            var c = (SampleTextBg as SolidColorBrush).Color;
                            if (Hightlight.Color.A != c.A || Hightlight.Color.B != c.B || Hightlight.Color.G != c.G || Hightlight.Color.R != c.R)
                                Hightlight.Color = (SampleTextBg as SolidColorBrush).Color;

                        }
                        break;

                    case AnnotArgsType.AnnotUnderline:
                        {
                            var Underline = Annot as TextUnderlineAnnotArgs;

                            if (Underline.Transparency != AnnotOpacity)
                                Underline.Transparency = AnnotOpacity;

                            var c = (SelectColor as SolidColorBrush).Color;
                            if (Underline.Color.A != c.A || Underline.Color.B != c.B || Underline.Color.G != c.G || Underline.Color.R != c.R)
                                Underline.Color = (SelectColor as SolidColorBrush).Color;
                        }
                        break;

                    case AnnotArgsType.AnnotSquiggly:
                        {
                            var Squiggly = Annot as TextSquigglyAnnotArgs;
                            if (Squiggly.Transparency != AnnotOpacity)
                                AnnotOpacity = Squiggly.Transparency;

                            var c = (SelectColor as SolidColorBrush).Color;
                            if (Squiggly.Color.A != c.A || Squiggly.Color.B != c.B || Squiggly.Color.G != c.G || Squiggly.Color.R != c.R)
                                Squiggly.Color = (SelectColor as SolidColorBrush).Color;
                        }
                        break;

                    case AnnotArgsType.AnnotStrikeout:
                        {
                            var Strikeout = Annot as TextStrikeoutAnnotArgs;
                            if (Strikeout.Transparency != AnnotOpacity)
                                Strikeout.Transparency = AnnotOpacity;

                            var c = (SelectColor as SolidColorBrush).Color;
                            if (Strikeout.Color.A != c.A || Strikeout.Color.B != c.B || Strikeout.Color.G != c.G || Strikeout.Color.R != c.R)
                                Strikeout.Color = (SelectColor as SolidColorBrush).Color;
                        }
                        break;

                }
            }
        }

    }
}
